"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.RequestManager = void 0;
const constants_1 = require("../constants/constants");
const undici_1 = require("undici");
const generateDS_1 = require("./generateDS");
const error_1 = require("./error");
const error_2 = require("../constants/error");
const debug_1 = require("./debug");
/**
 * @description Creates a new instance for the request
 */
class RequestManager {
    constructor(option) {
        Object.defineProperty(this, "baseURL", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "language", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "headers", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        this.baseURL = option?.route || constants_1.API_URL.Genshin_Battle;
        this.language = option?.language || "en-us" /* Language.EnglishUS */;
        this.headers = {
            "x-rpc-language": this.language,
        };
        if (option?.withUA) {
            this.headers["User-Agent"] = constants_1.UA;
        }
        if (option?.withExtraHeaders) {
            this.headers["x-rpc-app_version"] = "1.5.0";
            this.headers["x-rpc-client_type"] = "5";
        }
        if (option?.withDS) {
            this.headers.DS = (0, generateDS_1.generate_dynamic_secret)();
        }
    }
    _debug(message) {
        if ((0, debug_1.isDebug)()) {
            if (Array.isArray(message)) {
                console.log(`[DEBUG] ${message.join("\n[DEBUG] ")}`);
            }
            else {
                console.log(`[DEBUG] ${message}`);
            }
        }
    }
    async get(url, headers, params) {
        const URL = `${this.baseURL}${url}`;
        const requestHeaders = {
            ...this.headers,
            ...headers,
        };
        this._debug([`GET ${URL}`, `Headers: ${JSON.stringify(requestHeaders)}`, `Params: ${JSON.stringify(params)}`]);
        const { body: res, statusCode } = await (0, undici_1.request)(URL, {
            method: "GET",
            headers: requestHeaders,
            query: params,
        });
        if (statusCode !== 200) {
            this._debug(`Request failed with status code ${statusCode}`);
            throw new Error(`Request failed with status code ${statusCode}`);
        }
        const resData = await res.json();
        this._debug([
            `Retcode: ${resData.retcode}`,
            `Message: ${resData.message}`,
            `Data: ${JSON.stringify(resData.data)}`,
        ]);
        if (resData.retcode !== 0) {
            const description = (0, error_2.getErrorByRetcode)(resData.retcode);
            this._debug(`Error: ${description}`);
            throw new error_1.APIERROR(`${resData.message}`, resData.retcode, description);
        }
        return resData;
    }
    async post(url, headers, data, params) {
        const URL = `${this.baseURL}${url}`;
        const requestHeaders = {
            "content-type": "application/json",
            ...this.headers,
            ...headers,
        };
        const body = JSON.stringify(data);
        this._debug([
            `POST ${URL}`,
            `Headers: ${JSON.stringify(requestHeaders)}`,
            `Params: ${JSON.stringify(params)}`,
            `Body: ${body}`,
        ]);
        const { body: res, statusCode } = await (0, undici_1.request)(URL, {
            method: "POST",
            headers: requestHeaders,
            body: body,
            query: params,
        });
        if (statusCode !== 200) {
            this._debug(`Request failed with status code ${statusCode}`);
            throw new Error(`Request failed with status code ${statusCode}`);
        }
        const resData = await res.json();
        this._debug([
            `Retcode: ${resData.retcode}`,
            `Message: ${resData.message}`,
            `Data: ${JSON.stringify(resData.data)}`,
        ]);
        if (resData.retcode !== 0 && resData.retcode !== -5003) {
            const description = (0, error_2.getErrorByRetcode)(resData.retcode);
            this._debug(`Error: ${description}`);
            throw new error_1.APIERROR(`${resData.message}`, resData.retcode, description);
        }
        return resData;
    }
}
exports.RequestManager = RequestManager;
